/*
 * File access convenience functions.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <ipc/cap_alloc.h>
#include <ipc/mem_ipc.h>

#include "dataspace_client.h"
#include "opener_client.h"
#include "opener_context_client.h"
#include "mapped_file_client.h"

#include "file.h"



/* Release resources for the given file. */

void file_close(file_t *file)
{
  if (l4_is_valid_cap(file->ref))
    ipc_cap_free_um(file->ref);

  if (file->memory != NULL)
    ipc_detach_dataspace(file->memory);

  file_init(file);
}

/* Initialise a file structure for a context obtained from the given server. */

long file_context(file_t *file, l4_cap_idx_t server)
{
  client_Opener opener(server);
  unsigned long size, flags;
  long err;

  file_init(file);
  err = opener.context(&file->ref);

  if (err)
    return err;

  client_Dataspace context_ds(file->ref);

  err = context_ds.info(&size, &flags);

  if (err)
    return err;

  return ipc_attach_dataspace(file->ref, size, (void **) &file->memory);
}

/* Initialise the given file structure. */

void file_init(file_t *file)
{
  file->memory = NULL;
  file->ref = L4_INVALID_CAP;
}

/* Map a region of the given file to a memory region. */

long file_mmap(file_t *file, offset_t position, offset_t length)
{
  client_MappedFile mapped_file(file->ref);
  long err = mapped_file.mmap(position, length, &file->start_pos, &file->end_pos, &file->data_end);

  if (err)
    return err;

  return ipc_attach_dataspace(file->ref, file->end_pos - file->start_pos, (void **) &file->memory);
}

/* Open a file using the given structure and context reference. */

long file_open(file_t *file, l4_cap_idx_t context_ref)
{
  client_OpenerContext context(context_ref);
  file_init(file);
  return context.open(L4_FPAGE_RW, &file->size, &file->ref);
}

/* Return the amount of data in the mapped region for the given file. */

offset_t file_span(file_t *file)
{
  offset_t size = file->end_pos - file->start_pos;

  return (file->data_end < size) ? file->data_end : size;
}

// vim: tabstop=2 expandtab shiftwidth=2
